/*
 * Copyright (c) 2006, David Peterson
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of "randombits.org" nor the names of its contributors
 *       may be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.randombits.confluence.filtering.criteria.content;

import java.util.Collection;
import java.util.Set;

import com.atlassian.confluence.core.ConfluenceEntityObject;
import com.atlassian.confluence.core.ContentEntityObject;
import com.atlassian.confluence.core.PageContentEntityObject;
import com.atlassian.confluence.core.SpaceContentEntityObject;
import com.atlassian.confluence.pages.Page;
import com.atlassian.confluence.pages.Attachment;
import com.atlassian.confluence.spaces.Space;
import com.atlassian.confluence.links.linktypes.AbstractContentEntityLink;

import org.randombits.confluence.filtering.criteria.CriteriaException;
import org.randombits.confluence.filtering.criteria.Criterion;
import org.randombits.confluence.filtering.criteria.CriterionInterpreter;
import org.randombits.confluence.filtering.criteria.SourceCriterion;
import org.randombits.confluence.support.LinkAssistant;

/**
 * Checks if the supplied object matches the scope criterion.
 * 
 * @author David Peterson
 */
public class ContentScopeCriterion implements SourceCriterion<ContentEntityObject> {
    public static class Interpreter implements CriterionInterpreter {
        /**
         * This marks the delimitation between the content and the scope.
         */
        public static final char SCOPE_DELIMITER = '>';

        private ContentEntityObject content;

        public Interpreter() {
            this( null );
        }

        public Interpreter( ContentEntityObject content ) {
            this.content = content;
        }

        public Criterion createCriterion( String value ) throws CriteriaException {
            int scopeIndex = value.indexOf( SCOPE_DELIMITER );
            int scope = 0;

            if ( scopeIndex >= 0 ) {
                String scopeString = value.substring( scopeIndex + 1 ).trim().toLowerCase();
                value = value.substring( 0, scopeIndex ).trim();

                if ( "ancestors".equals( scopeString ) ) {
                    scope += ContentScopeCriterion.ANCESTORS_SCOPE;
                } else if ( "children".equals( scopeString ) ) {
                    scope += ContentScopeCriterion.CHILDREN_SCOPE;
                } else if ( "descendents".equals( scopeString ) || "descendants".equals( scopeString ) ) {
                    scope += ContentScopeCriterion.DESCENDENTS_SCOPE;
                } else {
                    throw new CriteriaException( "Unsupported scope: " + scopeString );
                }
            } else {
                scope = ContentScopeCriterion.SELF_SCOPE;
            }

            ContentEntityObject context = null;

            if ( "@self".equals( value ) ) {
                context = this.content;
            } else if ( "@parent".equals( value ) ) {
                if ( content instanceof Page ) {
                    context = ( ( Page ) content ).getParent();
                } else
                    throw new CriteriaException( "@parent can only be used on pages." );
            } else if ( "@home".equals( value ) ) {
                if ( content instanceof SpaceContentEntityObject ) {
                    Space space = ( ( SpaceContentEntityObject ) content ).getSpace();
                    if ( space != null )
                        context = space.getHomePage();
                }
            } else {
                ConfluenceEntityObject entity = LinkAssistant.getInstance().getLinkedEntity(
                        content.toPageContext(), value );
                if ( entity instanceof ContentEntityObject )
                    context = ( ContentEntityObject ) entity;
            }

            if ( context != null )
                return new ContentScopeCriterion( context, scope );
            else
                throw new CriteriaException( "Unable to locate the specified content: '" + value + "'" );
        }
    };

    /**
     * The scope
     */
    public static final int SELF_SCOPE = 1;

    public static final int CHILDREN_SCOPE = 2;

    public static final int DESCENDENTS_SCOPE = 4;

    public static final int ANCESTORS_SCOPE = 8;

    private ContentEntityObject rootContent;

    private int scope;

    public ContentScopeCriterion( ContentEntityObject ceo ) {
        this( ceo, SELF_SCOPE );
    }

    public ContentScopeCriterion( ContentEntityObject ceo, int scope ) {
        rootContent = ceo;
        this.scope = scope;
    }

    public boolean matches( Object object ) {
        ContentEntityObject ceo = null;

        if ( object instanceof PageContentEntityObject ) {
            ceo = ( ( PageContentEntityObject ) object ).getPage();
        } else if ( object instanceof ContentEntityObject ) {
            ceo = ( ContentEntityObject ) object;
        } else if ( object instanceof Attachment ) {
            ceo = ( ( Attachment ) object ).getContent();
        } else if ( object instanceof AbstractContentEntityLink ) {
            ceo = ( ( AbstractContentEntityLink ) object ).getDestinationContent();
        }

        if ( ceo != null ) {
            if ( hasScope( SELF_SCOPE ) ) {
                if ( rootContent.equals( ceo ) )
                    return true;
            }

            if ( scope != SELF_SCOPE && rootContent instanceof Page && ceo instanceof Page ) {
                // there are other scopes set
                Page page = ( Page ) ceo;

                if ( hasScope( CHILDREN_SCOPE ) ) {
                    if ( rootContent.equals( page.getParent() ) )
                        return true;
                }

                if ( hasScope( DESCENDENTS_SCOPE ) ) {
                    if ( isAncestor( page, ( Page ) rootContent ) )
                        return true;
                }

                if ( hasScope( ANCESTORS_SCOPE ) ) {
                    if ( isAncestor( ( Page ) rootContent, page ) )
                        return true;
                }
            }
        }
        return false;
    }

    private boolean hasScope( int checkScope ) {
        return ( scope & checkScope ) != 0;
    }

    private boolean isAncestor( Page page, Page ancestor ) {
        page = page.getParent();
        while ( page != null && !page.equals( ancestor ) )
            page = page.getParent();

        return page != null;
    }

    @Override
    public String toString() {
        return "{content scope: "
                + rootContent
                + ( scope == 0 ? "" : " >" + ( hasScope( CHILDREN_SCOPE ) ? " children" : "" )
                        + ( hasScope( DESCENDENTS_SCOPE ) ? " descendents" : "" )
                        + ( hasScope( ANCESTORS_SCOPE ) ? " ancestors" : "" ) ) + "}";
    }

    public Collection<ContentEntityObject> getMatchingValues() {
        Set<ContentEntityObject> values = new java.util.HashSet<ContentEntityObject>();

        if ( rootContent != null && scope == 0 ) {
            if ( hasScope( SELF_SCOPE ) ) {
                values.add( rootContent );
            }

            if ( rootContent instanceof Page ) {
                Page page = ( Page ) rootContent;
                if ( hasScope( DESCENDENTS_SCOPE ) ) {
                    values.addAll( page.getDescendents() );
                } else if ( hasScope( CHILDREN_SCOPE ) ) {
                    values.addAll( page.getChildren() );
                }

                if ( hasScope( ANCESTORS_SCOPE ) ) {
                    values.addAll( page.getAncestors() );
                }
            }
        }

        return values;
    }

    public Weight getWeight() {
        return Weight.MEDIUM;
    }
}
